"""Common stuff used by multiple parts of OGS Container Maker"""

from __future__ import absolute_import

import sys

from ogscm.common import package_manager

g_package_manager = package_manager.SYSTEM

def set_package_manager(pm):
  this = sys.modules[__name__]

  if pm == 'conan':
    this.g_package_manager = package_manager.CONAN
  elif pm == 'spack':
    this.g_package_manager = package_manager.SPACK
  elif pm == 'easybuild':
    this.g_package_manager = package_manager.EASYBUILD
  elif pm == 'guix':
    this.g_package_manager = package_manager.GUIX
  elif pm == 'system':
    this.g_package_manager = package_manager.SYSTEM
  elif pm == 'off':
    this.g_package_manager = package_manager.OFF
  else:
    RuntimeError('Invalid package manager!')



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""Eigen building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.building_blocks.generic_cmake import generic_cmake


class eigen(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm,
            hpccm.templates.tar, hpccm.templates.wget):
    """The `eigen` building block downloads and installs the
    [Eigen](http://eigen.tuxfamily.org) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/eigen`.

    version: The version of Eigen source to download.  The default
    value is `3.3.4`.

    # Examples

    ```python
    eigen(version='3.2.9')
    ```

    """
    def __init__(self, **kwargs):
        """Initialize building block"""
        super(eigen, self).__init__()

        self.__baseurl = kwargs.get('baseurl',
                                    'http://bitbucket.org/eigen/eigen/get')
        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/eigen')
        self.__version = kwargs.get('version', '3.3.4')

        self.__instructions()

    def __instructions(self):
        """String representation of the building block"""
        self += comment('Eigen version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += generic_cmake(directory='eigen-eigen-*'.format(self.__version),
                              prefix=self.__prefix,
                              url='{0}/{1}.tar.gz'.format(
                                  self.__baseurl, self.__version))
        self += environment(variables={
            'Eigen3_ROOT': self.__prefix,
            'Eigen3_DIR': self.__prefix
        })

        # No runtime



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Package manager spack building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives import copy
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell
import hpccm.templates.wget


class pm_spack(bb_base, hpccm.templates.git):
    """Package manager spack building block"""

    def __init__(self, **kwargs):
        super(pm_spack, self).__init__()


        self.__ospackages = kwargs.get('ospackages', [])
        self.__packages = kwargs.get('packages', [])
        self.__repo = kwargs.get('repo', 'https://github.com/spack/spack')
        self.__branch = kwargs.get('branch', 'devel')

        self.__commands = []  # Filled in by __setup()
        self.__environment_variables = {}
        self.__labels = {}


        self.__wd = '/var/tmp'  # working directory

        self.__setup()

        self.__instructions()


    def __instructions(self):
        self += comment(__doc__, reformat=False)
        self += packages(ospackages=self.__ospackages)
        self += packages(yum=['xz'], apt=['xz-utils'])
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)
        if self.__labels:
            self += label(metadata=self.__labels)
        if self.__packages:
            self += copy(src='files/spack/packages.yml', dest='/etc/spack/packages.yaml')
            install_cmds = []
            for package in self.__packages:
                install_cmds.append('/opt/spack/bin/spack install {0}'.format(package))
            install_cmds.append('/opt/spack/bin/spack clean --all')
            self += shell(commands=install_cmds)


    def __setup(self):
        self.__ospackages.extend(['patch', 'less', 'curl', 'bzip2'])
        self.__commands.extend([
            self.clone_step(repository=self.__repo, branch=self.__branch,
                            path='/opt'),
            '/opt/spack/bin/spack bootstrap',
            # TODO: There is no init system inside the container -> files are
            #       not sourced!
            'ln -s /opt/spack/share/spack/setup-env.sh /etc/profile.d/spack.sh',
            'ln -s /opt/spack/share/spack/spack-completion.bash /etc/profile.d',
            '/opt/spack/bin/spack clean --all'
        ])

        # Environment
        self.__environment_variables['PATH'] = '/opt/spack/bin:$PATH'
        self.__environment_variables['FORCE_UNSAFE_CONFIGURE'] = '1'

        # Labels
        self.__labels['org.opengeosys.pm'] = 'spack'


    def runtime(self, _from='0'):
        """Install the runtime from a full build in a previous stage.  In this
           case there is no difference between the runtime and the
           full build."""
        return str(self)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""OGS base building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import hpccm.config

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.building_blocks.pip import pip
from hpccm.building_blocks.python import python
from hpccm.common import linux_distro
from hpccm.primitives.comment import comment
from hpccm.primitives.shell import shell


class ogs_base(bb_base):
    """OGS base building block"""
    def __init__(self, **kwargs):
        """Initialize building block"""
        super(ogs_base, self).__init__()

        self.__ospackages = kwargs.get('ospackages', [])

        self.__commands = []  # Filled in by __setup()
        self.__wd = '/var/tmp'  # working directory

        self.__setup()

        self.__instructions()

    def __instructions(self):
        """String representation of the building block"""
        self += comment(__doc__, reformat=False)
        self += python(devel=True, python2=False)
        self += pip(pip='pip3',
                    packages=['virtualenv', 'pre-commit', 'cmake-format'])
        self += packages(ospackages=self.__ospackages,
                         apt_ppas=['ppa:git-core/ppa'],
                         epel=True)
        self += shell(commands=self.__commands)

    def __setup(self):
        self.__ospackages.extend(['git', 'git-lfs', 'ninja-build'])

        dist = 'deb'
        if hpccm.config.g_linux_distro == linux_distro.CENTOS:
            dist = 'rpm'
        if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
            self.__commands.extend([
                "apt-get update",
                # Workaround for https://github.com/git-lfs/git-lfs/issues/3474
                "apt-get install -y dirmngr --install-recommends",
                "apt-key adv --keyserver keyserver.ubuntu.com --recv-keys "
                "6B05F25D762E3157"
            ])
        self.__commands.append(
            "curl -s https://packagecloud.io/install/repositories/github/"
            "git-lfs/script.{0}.sh | bash".format(dist))

        if hpccm.config.g_ctype == hpccm.container_type.SINGULARITY:
            self.__ospackages.append('locales')
            self.__commands.extend([
                'echo "LC_ALL=en_US.UTF-8" >> /etc/environment',
                'echo "en_US.UTF-8 UTF-8" >> /etc/locale.gen',
                'echo "LANG=en_US.UTF-8" > /etc/locale.conf',
                'locale-gen en_US.UTF-8'
            ])

        self.__commands.append('git lfs install')

        # Common directories
        self.__commands.append(
            'mkdir -p /apps /scratch /lustre /work /projects /data')

    def runtime(self, _from='0'):
        p = python(devel=True, python2=False)
        instructions = [
            comment(__doc__, reformat=False),
            p.runtime(),
            shell(commands=['mkdir -p /apps /scratch /lustre /work /projects'])
        ]

        return '\n'.join(str(x) for x in instructions)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Package manager easybuild building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from ogscm.building_blocks import lmod
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell


class pm_easybuild(object):
  """Package manager easybuild building block"""

  def __init__(self, **kwargs):
    """Initialize building block"""

    # Trouble getting MRO with kwargs working correctly, so just call
    # the parent class constructors manually for now.
    # super(python, self).__init__(**kwargs)

    self.__ospackages = kwargs.get('ospackages', [])

    self.__commands = []  # Filled in by __setup()
    self.__wd = '/var/tmp'  # working directory

    self.__setup()

  def __str__(self):
    """String representation of the building block"""
    instructions = [
      comment(__doc__, reformat=False),
      packages(ospackages=self.__ospackages),
      packages(yum=['xz'], apt=['xz-utils']),
      lmod(version='7.8.6'), shell(commands=self.__commands),
      environment(variables={
        'MODULEPATH': '/opt/easybuild/modules/all:/home/easybuild/.local/'
                      'easybuild/modules/all:$MODULEPATH',
        'FORCE_UNSAFE_CONFIGURE': '1',
        # https://github.com/docker-library/python/blob/edde349541e11f66dcc79cde1674317d065ddbdd/3.6/Dockerfile#L8
        'LANG': 'C.UTF-8'
      }), label(metadata={'PACKAGE_MANAGER': 'easybuild'})
    ]
    # Without the FORCE_UNSAFE_CONFIGURE env var some spack package
    # installations may fail due to running as root.

    return '\n'.join(str(x) for x in instructions)

  def __setup(self):
    self.__ospackages.extend([
      'build-essential', 'bzip2', 'file', 'git', 'gzip',
      'libssl-dev', 'libtool', 'make', 'openssh-client',
      'patch', 'python-pip', 'python-setuptools', 'rsh-client',
      'tar', 'wget', 'unzip'
    ])
    self.__commands.extend([
      'useradd -m easybuild',
      'mkdir -p /opt/easybuild',
      'chown easybuild:easybuild /opt/easybuild',
      'easy_install easybuild==3.7.0'
    ])

  def runtime(self, _from='0'):
    """Install the runtime from a full build in a previous stage.  In this
       case there is no difference between the runtime and the
       full build."""
    return str(self)

  def install(self, ospackages=None, configs=None):
    instructions = []
    if ospackages:
      instructions.append(packages(ospackages=ospackages))
    commands = []
    for config in configs:
      commands.append(
        'runuser easybuild -l -c "eb {} -r --installpath /opt/easybuild"'.format(config)
      )
    instructions.append(shell(commands=commands))

    return '\n'.join(str(x) for x in instructions)



#!/usr/bin/env python
import subprocess

subprocess.run(f"python ogscm/cli.py --pm system --cvode "
               "--ompi off 2.1.6 3.1.4 4.0.1",
               shell=True)

subprocess.run(f"python ogscm/cli.py --file Dockerfile.gcc.full --jenkins "
               "--cppcheck --docs --gcovr --compiler_version 8",
               shell=True)
subprocess.run(f"python ogscm/cli.py --file Dockerfile.gcc.gui --jenkins "
               "--cppcheck --gui --gcovr --compiler_version 8",
               shell=True)
subprocess.run(f"python ogscm/cli.py --file Dockerfile.clang.full "
               "--base_image ubuntu:18.04 --compiler clang --jenkins "
               "--iwyy",
               shell=True)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Jenkins Node building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import logging  # pylint: disable=unused-import
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.shell import shell
from hpccm.primitives.user import user
from hpccm.primitives.workdir import workdir


class jenkins_node(bb_base, hpccm.templates.rm, hpccm.templates.tar,
                   hpccm.templates.wget):
  """Jenkins Node building block"""

  def __init__(self, **kwargs):
    """Initialize building block"""
    super(jenkins_node, self).__init__()

    self.__commands = []  # Filled in by __setup()
    self.__wd = '/var/tmp'  # working directory

    self.__instructions()

  def __instructions(self):
    self += comment('Jenkins node')
    self += packages(ospackages=['rsync', 'openssh-client', 'sudo'])
    self += shell(commands=[
        'groupadd --gid 1001 jenkins || true',
        'adduser --uid 500 --gid 1001 --disabled-password --gecos "" jenkins',
        'echo "jenkins ALL=(ALL) NOPASSWD:ALL" >> /etc/sudoers',
        'echo "jenkins:jenkins" | chpasswd'
    ])
    self += user(user='jenkins')
    self += workdir(directory='/home/jenkins')

    logging.warning("Changed user to jenkins!")


  def runtime(self, _from='0'):
    """Install the runtime from a full build in a previous stage.  In this
       case there is no difference between the runtime and the
       full build."""
    return str(self)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""self Arguments"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import argparse
import math
import multiprocessing

from ogscm.version import __version__


class Cli_Args(argparse.ArgumentParser):
    def __init__(self, **kwargs):
        """Initialize"""
        argparse.ArgumentParser.__init__(self)
        self.formatter_class = argparse.ArgumentDefaultsHelpFormatter

        self.add_argument(
            '--version',
            action='version',
            version='%(prog)s {version}'.format(version=__version__))
        self.add_argument('--out',
                          type=str,
                          default='_out',
                          help='Output directory')
        self.add_argument('--file',
                          type=str,
                          default='',
                          help='Overwrite output recipe file name')
        self.add_argument('--print',
                          '-P',
                          dest='print',
                          action='store_true',
                          help='Print the definition to stdout')
        options_g = self.add_argument_group(
            'Combinatorial options',
            'All combinations of the given options will '
            'be generated')
        options_g.add_argument('--format',
                               nargs='*',
                               type=str,
                               choices=['docker', 'singularity'],
                               default=['docker'])
        options_g.add_argument('--pm',
                               nargs='*',
                               type=str,
                               choices=['system', 'conan', 'spack', 'off'],
                               default=['conan'],
                               help='Package manager to install third-party '
                               'dependencies')
        options_g.add_argument(
            '--ompi',
            nargs='*',
            type=str,
            default=['off'],
            help='OpenMPI version, e.g. 2.1.1, 2.1.5, 3.0.1, 3.1.2')
        options_g.add_argument(
            '--ogs',
            nargs='*',
            type=str,
            default=['ufz/ogs@master'],
            help='OGS GitHub repo in the form \'user/repo@branch\' '
            'OR \'user/repo@@commit\' to checkout a specific commit '
            'OR a path to a local subdirectory to the git cloned OGS sources '
            'OR \'off\' to disable OGS building')
        options_g.add_argument(
            '--cmake_args',
            nargs='*',
            type=str,
            default=[''],
            help='CMake argument sets have to be quoted and **must**'
            ' start with a space. e.g. --cmake_args \' -DFIRST='
            'TRUE -DFOO=BAR\' \' -DSECOND=TRUE\'')
        build_g = self.add_argument_group('Image build options')
        build_g.add_argument('--build',
                             '-B',
                             dest='build',
                             action='store_true',
                             help='Build the images from the definition files')
        build_g.add_argument('--upload',
                             '-U',
                             dest='upload',
                             action='store_true',
                             help='Upload Docker image to registry')
        build_g.add_argument(
            '--registry',
            type=str,
            default='registry.opengeosys.org/ogs/ogs',
            help='The docker registry the image is tagged and '
            'uploaded to.')
        build_g.add_argument(
            '--tag',
            type=str,
            default='',
            help='The full docker image tag. Overwrites --registry.')
        build_g.add_argument('--convert',
                             '-C',
                             dest='convert',
                             action='store_true',
                             help='Convert Docker image to Singularity image')
        build_g.add_argument(
            '--runtime-only',
            '-R',
            dest='runtime_only',
            action='store_true',
            help='Generate multi-stage Dockerfiles for small runtime '
            'images')
        build_g.add_argument(
            '--ccache',
            dest='ccache',
            action='store_true',
            help='Enables ccache build caching.'
        )
        build_g.add_argument(
            '--parallel',
            '-j',
            type=str,
            default=math.ceil(multiprocessing.cpu_count() / 2),
            help='The number of cores to use for compilation.'
        )
        switches_g = self.add_argument_group('Additional options')
        switches_g.add_argument('--base_image',
                                type=str,
                                default='ubuntu:19.04',
                                help='The base image.')
        switches_g.add_argument(
            '--compiler',
            type=str,
            default='gcc',
            help='The compiler to use. Possible options: off, gcc, clang')
        switches_g.add_argument('--compiler_version',
                                type=str,
                                default='',
                                help='Compiler version.')
        switches_g.add_argument('--gui',
                                dest='gui',
                                action='store_true',
                                help='Builds the GUI (Data Explorer)')
        switches_g.add_argument(
            '--docs',
            dest='docs',
            action='store_true',
            help='Setup documentation requirements (Doxygen)')
        switches_g.add_argument('--jenkins',
                                dest='jenkins',
                                action='store_true',
                                help='Setup Jenkins slave')
        switches_g.add_argument('--gitlab',
                                dest='gitlab',
                                action='store_true',
                                help='Setup for GitLab-CI')
        switches_g.add_argument('--cvode',
                                dest='cvode',
                                action='store_true',
                                help='Install and configure with cvode')
        switches_g.add_argument('--cppcheck',
                                dest='cppcheck',
                                action='store_true',
                                help='Install cppcheck')
        switches_g.add_argument('--iwyy',
                                dest='iwyy',
                                action='store_true',
                                help='Install include-what-you-use')
        switches_g.add_argument('--gcovr',
                                dest='gcovr',
                                action='store_true',
                                help='Install gcovr')
        switches_g.add_argument('--mpi_benchmarks',
                                dest='mpi_benchmarks',
                                action='store_true',
                                help='Installs OSU MPI '
                                'benchmarks as scif app and mpi_bw, mpi_ring,'
                                'mpi_hello')
        switches_g.add_argument('--dev',
                                dest='dev',
                                action='store_true',
                                help='Installs development tools (vim, gdb)')
        switches_g.add_argument('--insitu',
                                dest='insitu',
                                action='store_true',
                                help='Builds with insitu capabilities')
        switches_g.add_argument('--pip',
                               nargs='*',
                               type=str,
                               default=[],
                               metavar='package',
                               help='Install additional Python packages')
        maint_g = self.add_argument_group('Maintenance')
        maint_g.add_argument(
            '--clean',
            dest='cleanup',
            action='store_true',
            help='Cleans up generated files in default directories.')
        deploy_g = self.add_argument_group('Image deployment')
        deploy_g.add_argument(
            '--deploy',
            '-D',
            nargs='?',
            const='ALL',
            type=str,
            default='',
            help=
            'Deploys to all configured hosts (in config/deploy_hosts.yml) with no additional arguments or to the specified host. Implies --build and --convert arguments.'
        )



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""cppcheck building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.building_blocks.generic_cmake import generic_cmake


class cppcheck(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm,
               hpccm.templates.tar, hpccm.templates.wget):
    """The `cvode` building block downloads and installs the
    [cppcheck](https://computation.llnl.gov/projects/sundials/cvode) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/cppcheck`.

    version: The version of cppcheck source to download.  The default
    value is `1.83`.
    """
    def __init__(self, **kwargs):
        """Initialize building block"""
        super(cppcheck, self).__init__()

        self.__baseurl = kwargs.get(
            'baseurl', 'https://github.com/danmar/cppcheck/archive')
        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/cppcheck')
        self.__version = kwargs.get('version', '1.87')

        self.__instructions()

    def __instructions(self):
        self += comment('cppcheck version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += generic_cmake(cmake_opts=['-D CMAKE_BUILD_TYPE=Release'],
                              directory='cppcheck-{}'.format(self.__version),
                              prefix=self.__prefix,
                              url='{0}/{1}.tar.gz'.format(
                                  self.__baseurl, self.__version))
        self += environment(
            variables={'PATH': '{0}/bin:$PATH'.format(self.__prefix)})

        # No runtime



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""PETSc building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell
from hpccm.toolchain import toolchain
from hpccm.building_blocks.generic_autotools import generic_autotools


class petsc(bb_base, hpccm.templates.ConfigureMake, hpccm.templates.ldconfig,
            hpccm.templates.rm, hpccm.templates.tar, hpccm.templates.wget):
    """The `PETSc` building block downloads and installs the
    [VTK](https://vtk.org/) component.

    Requires python3.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/petsc`.

    version: The version of PETSc source to download.  The default
    value is `3.8.4`.

    # Examples

    ```python
    petsc(version='3.8.1')
    ```

    """
    def __init__(self, **kwargs):
        super(petsc, self).__init__(**kwargs)

        self.__ospackages = kwargs.get('ospackages', [])
        self.parallel = 1
        self.__prefix = kwargs.get('prefix', '/usr/local/petsc')
        self.__toolchain = toolchain(CC='mpicc', CXX='mpicxx')
        self.configure_opts = kwargs.get('configure_opts', [])
        self.__version = kwargs.get('version', '3.8.4')
        self.__wd = '/var/tmp'  # working directory
        self.__baseurl = kwargs.get(
            'baseurl', 'http://ftp.mcs.anl.gov/pub/petsc/release-snapshots')
        self.__environment_variables = {}

        self.__instructions()

    def __instructions(self):
        self += comment('PETSc {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += generic_autotools(
            directory='petsc-{}'.format(self.__version),
            preconfigure=["sed -i -- 's/python/python3/g' configure"],
            prefix=self.__prefix,
            toolchain=self.__toolchain,
            url='{0}/petsc-lite-{1}.tar.gz'.format(self.__baseurl,
                                                   self.__version),
            configure_opts=[
                'CC={}'.format(self.__toolchain.CC),
                'CXX={}'.format(self.__toolchain.CXX), '--CFLAGS=\'-O3\'',
                '--CXXFLAGS=\'-O3\'', '--FFLAGS=\'-O3\'',
                '--with-debugging=no', '--with-fc=0',
                '--download-f2cblaslapack=1'
            ])
        self.__environment_variables['PETSC_DIR'] = self.__prefix
        # Set library path
        libpath = os.path.join(self.__prefix, 'lib')
        if self.ldconfig:
            self += shell(commands=[self.ldcache_step(directory=libpath)])
        else:
            self.__environment_variables[
                'LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(libpath)

        self += environment(variables=self.__environment_variables)
        self += label(metadata={'petsc.version': self.__version})

    def runtime(self, _from='0'):
        instructions = []
        instructions.append(comment('PETSc {}'.format(self.__version)))
        instructions.append(
            copy(_from=_from, src=self.__prefix, dest=self.__prefix))

        if self.ldconfig:
            libpath = os.path.join(self.__prefix, 'lib')
            instructions.append(
                shell(commands=[self.ldcache_step(directory=libpath)]))

        instructions.append(
            environment(variables=self.__environment_variables))
        instructions.append(label(metadata={'petsc.version': self.__version}))
        return '\n'.join(str(x) for x in instructions)




# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""Container info"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import hashlib
import json
import os
import re
import requests
import shutil
import subprocess

from ogscm import config


class container_info():
    def __init__(self, args_iter, args):
        """Initialize container info"""
        self.ogsdir = False
        self.outdir = ''
        self.definition_file = ''
        self.images_out_dir = ''
        self.img_file = ''
        self.commit_hash = ''
        self.repo = ''
        self.branch = ''
        self.git_version = ''

        container_format = args_iter[0]
        ogs_version = args_iter[1]
        ompi = args_iter[3]
        cmake_args = args_iter[4].strip().split(' ')
        name_start = 'gcc'

        branch_is_release = False
        if ogs_version != 'off':
            if os.path.isdir(ogs_version):
                self.repo = 'local'
                self.commit_hash = subprocess.run(
                    ['cd {} && git rev-parse HEAD'.format(ogs_version)],
                    capture_output=True,
                    text=True,
                    shell=True).stdout.rstrip()
                self.branch = subprocess.run([
                    'cd {} && git branch | grep \* | cut -d \' \' -f2'.format(
                        ogs_version)
                ],
                                             capture_output=True,
                                             text=True,
                                             shell=True).stdout
                self.git_version = subprocess.run(
                    ['cd {} && git describe --tags'.format(ogs_version)],
                    capture_output=True,
                    text=True,
                    shell=True).stdout[0]
            else:
                # Get git commit hash and construct image tag name
                self.repo, self.branch, *commit = ogs_version.split("@")
                if commit:
                    self.commit_hash = commit[0]
                    if self.branch == '':
                        self.branch = 'master'
                else:
                    if re.search(r'[\d.]+', self.branch):
                        branch_is_release = True
                    url = (
                        f"https://api.github.com/repos/{self.repo}/commits?sha={self.branch}"
                    )
                    response = requests.get(url)
                    response_data = json.loads(response.text)
                    self.commit_hash = response_data[0]['sha']
                    # ogs_tag = ogs_version.replace('/', '.').replace('@', '.')

            if branch_is_release:
                name_start = f'ogs-{self.branch}'
            else:
                name_start = f'ogs-{self.commit_hash[:8]}'
        else:
            if args.compiler == 'clang':
                name_start = 'clang'

        name_openmpi = 'serial'
        if ompi != 'off':
            name_openmpi = f"openmpi-{ompi}"

        if len(cmake_args) > 0:
            cmake_args_hash = hashlib.md5(
                ' '.join(cmake_args).encode('utf-8')).hexdigest()
            cmake_args_hash_short = cmake_args_hash[:8]

        # name_image = args.base_image.replace(':', '_')
        # Removed {name_image}/
        img_folder = (f"{name_start}/{name_openmpi}/"
                      f"{config.g_package_manager.name.lower()}")
        self.img_file = img_folder.replace("/", "-")
        if len(cmake_args) > 0:
            self.img_file += f'-cmake-{cmake_args_hash_short}'
        if args.gui:
            self.img_file += '-gui'
        if ogs_version != 'off' and not args.runtime_only:
            self.img_file += '-dev'

        if args.tag != '':
            self.tag = args.tag
        else:
            self.tag = f"{args.registry}/{self.img_file}:latest"

        if os.path.isdir(ogs_version):
            self.ogsdir = True

        if args.file != '':
            self.out_dir = args.out
            self.definition_file = args.file
        else:
            if self.ogsdir:
                self.out_dir = os.path.join(
                    ogs_version, f"{args.out}/{container_format}/{img_folder}")
            else:
                self.out_dir = f"{args.out}/{container_format}/{img_folder}"
            if len(cmake_args) > 0:
                self.out_dir += f'/cmake-{cmake_args_hash_short}'
            self.images_out_dir = f"{args.out}/images"
            self.definition_file = 'Dockerfile'
            if container_format == 'singularity':
                self.definition_file = 'Singularity.def'

    def make_dirs(self):
        if not os.path.exists(self.out_dir):
            os.makedirs(self.out_dir)  # For .scif files
        if self.images_out_dir and not os.path.exists(self.images_out_dir):
            os.makedirs(self.images_out_dir)

    def cleanup(self):
        shutil.rmtree(self.out_dir, ignore_errors=True)
        print('Cleaned up!')





# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""OSU benchmarks building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell
from hpccm.toolchain import toolchain


class osu_benchmarks(bb_base, hpccm.templates.ConfigureMake, hpccm.templates.rm,
                     hpccm.templates.tar, hpccm.templates.wget):
    """OSU benchmarks building block"""

    def __init__(self, **kwargs):
        super(osu_benchmarks, self).__init__()

        self.__ospackages = kwargs.get('ospackages', ['wget', 'tar'])
        self.prefix = kwargs.get('prefix', '/usr/local/osu-benchmarks')
        self.__version = kwargs.get('version', '5.4.2')
        self.__toolchain = toolchain(CC='mpicc', CXX='mpicxx')
        self.configure_opts = kwargs.get('configure_opts', [])
        self.__wd = '/var/tmp'  # working directory

        # Filled in by __setup():
        self.__commands = []
        self.__environment_variables = {}
        self.__labels = {}

        self.__setup()

        self.__instructions()

    def __instructions(self):
        self += comment('OSU benchmarks version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)
        if self.__labels:
            self += label(metadata=self.__labels)


    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
           self.__commands"""

        # Get the source
        directory = 'osu-micro-benchmarks-{}'.format(self.__version)
        tarball = '{}.tar.gz'.format(directory)
        self.__commands.append(self.download_step(
            url="http://mvapich.cse.ohio-state.edu/download/mvapich/{}".format(
                tarball),
            directory=self.__wd))
        self.__commands.append(self.untar_step(
            tarball=os.path.join(self.__wd, tarball), directory=self.__wd))

        # Configure, build, install
        self.__commands.append(self.configure_step(
            directory=os.path.join(self.__wd, directory),
            opts=[
                'CC={}'.format(self.__toolchain.CC),
                'CXX={}'.format(self.__toolchain.CXX)
            ]
        ))
        self.__commands.append(self.build_step())
        self.__commands.append(self.install_step())

        # Cleanup
        self.__commands.append(self.cleanup_step(
            items=[os.path.join(self.__wd, tarball),
                   os.path.join(self.__wd, directory)]))

        # Environment
        libexec_path = '{0}/libexec/osu-micro-benchmarks/mpi'.format(self.prefix)
        self.__environment_variables['PATH'] = '{0}/collective:{0}/one-sided:{0}/pt2pt:{0}/startup::$PATH'.format(libexec_path)

        # Labels
        self.__labels['osu.version'] = self.__version

    def runtime(self, _from='0'):
        instructions = []
        instructions.append(comment('OSU benchmarks version {}'.format(self.__version)))
        instructions.append(copy(_from=_from, src=self.prefix,
                                dest=self.prefix))
        if self.__environment_variables:
            instructions.append(environment(
                variables=self.__environment_variables))
        if self.__labels:
            instructions.append(label(metadata=self.__labels))
        return '\n'.join(str(x) for x in instructions)



#!/usr/bin/env python3

# OpenMPI versions:
#  Taurus: 1.8.8, 1.10.2, 2.1.0, 2.1.1, 3.0.0, 3.1.4, 4.0.1
#  Eve: 1.8.8, 1.10.2, 2.1.1, 4.0.0
#  --> 2.1.1
# https://easybuild.readthedocs.io/en/latest/Common-toolchains.html#common-toolchains-overview
# easybuild toolchain: 2017b (2.1.1), 2018a (2.1.2), 2018b (3.1.1)
import argparse
import itertools
import json
import os
import re
import requests
import subprocess
import sys
import yaml

from packaging import version

import hpccm
from hpccm import linux_distro
from hpccm.building_blocks import packages, mlnx_ofed, knem, ucx, openmpi, \
    boost, pip, scif, llvm, gnu, ofed, cmake, slurm_pmi2, pmix
from hpccm.primitives import baseimage, comment, user, environment, raw, \
    label, shell, copy

import ogscm
from ogscm.cli_args import Cli_Args
from ogscm.config import package_manager
from ogscm.container_info import container_info
from ogscm.version import __version__
from ogscm.building_blocks import ccache, cppcheck, cvode, eigen, iwyy, \
    jenkins_node, ogs_base, ogs, osu_benchmarks, petsc, pm_conan, vtk, \
    pm_spack, paraview


def main():  # pragma: no cover
    cli = Cli_Args()
    args = cli.parse_args()

    if args.jenkins:
        args.ogs = ['off']
    if args.deploy != '':
        args.build = True
        args.convert = True

    cwd = os.getcwd()

    c = list(
        itertools.product(args.format, args.ogs, args.pm, args.ompi,
                          args.cmake_args))
    if not args.print and not args.cleanup:
        print('Creating {} image definition(s)...'.format(len(c)))
    for build in c:
        __format = build[0]
        ogs_version = build[1]
        ogscm.config.set_package_manager(build[2])
        ompi = build[3]
        cmake_args = build[4].strip().split(' ')

        # args checking
        if len(c) > 1 and args.file != '':
            print(
                '--file can only be used when generating a single image definition'
            )
            quit(1)
        if ogs_version == 'off' and len(
                cmake_args) > 0 and cmake_args[0] != '':
            cmake_args = []
            print('--cmake_args cannot be used with --ogs off! Ignoring!')
        if __format == 'singularity':
            if args.runtime_only:
                args.runtime_only = False
                print(
                    '--runtime-only cannot be used with --format singularity! '
                    'Ignoring!')
            if args.upload:
                print('--upload cannot be used with --format singularity! '
                      'Ignoring!')
            if args.convert:
                print('--convert cannot be used with --format singularity! '
                      'Ignoring!')

        info = container_info(build, args)
        if args.cleanup:
            info.cleanup()
            exit(0)
        info.make_dirs()

        # Create definition
        hpccm.config.set_container_format(__format)

        # ------------------------------ recipe -------------------------------
        Stage0 = hpccm.Stage()
        Stage0 += raw(docker='# syntax=docker/dockerfile:experimental')

        if args.runtime_only:
            Stage0.name = 'build'
        Stage0 += baseimage(image=args.base_image, _as='build')

        Stage0 += comment(f"Generated with ogs-container-maker {__version__}",
                          reformat=False)

        Stage0 += packages(ospackages=['wget', 'tar', 'curl', 'make'])

        # base compiler
        if args.compiler != 'off':
            if args.compiler_version == '':
                if args.compiler == 'clang':
                    args.compiler_version = '8'
                else:
                    args.compiler_version = None  # Use default
            if args.compiler == 'clang':
                compiler = llvm(extra_repository=True,
                                version=args.compiler_version)
            else:
                compiler = gnu(fortran=False,
                               extra_repository=True,
                               version=args.compiler_version)
            toolchain = compiler.toolchain
            Stage0 += compiler
            # Upgrade stdc++ lib after installing new compiler
            # https://stackoverflow.com/a/46613656/80480
            if args.compiler == 'gcc' and args.compiler_version != None:
                Stage0 += packages(apt=['libstdc++6'])
            if args.compiler == 'clang':
                Stage0 += packages(
                    apt=[
                        "clang-tidy-{}".format(args.compiler_version),
                        "clang-format-{}".format(args.compiler_version)
                    ],
                    yum=[
                        "llvm-toolset-{}-clang-tools-extra".format(
                            args.compiler_version)
                    ])

        # Prepare runtime stage
        Stage1 = hpccm.Stage()
        Stage1.baseimage(image=args.base_image)

        # Install scif in all stages
        Stage0 += pip(packages=['scif'], pip='pip3')
        Stage1 += pip(packages=['scif'], pip='pip3')

        if ompi != 'off':
            mpicc = object
            if False:  # eve:
                # Stage0 += ofed() OR mlnx_ofed(); is installed later on from debian archive
                # Stage0 += knem()
                Stage0 += ucx(version='1.5.1',
                              cuda=False)  #  knem='/usr/local/knem'
                Stage0 += packages(ospackages=['libpmi2-0-dev'
                                               ])  # req. for --with-pmi
                # req. for --with-psm2
                Stage0 += packages(ospackages=['libnuma1'])
                psm_deb_url = 'http://snapshot.debian.org/archive/debian/20181231T220010Z/pool/main'
                psm2_version = '11.2.68-4'
                Stage0 += shell(commands=[
                    'cd /tmp',
                    f'wget -nv {psm_deb_url}/libp/libpsm2/libpsm2-2_{psm2_version}_amd64.deb',
                    f'wget -nv {psm_deb_url}/libp/libpsm2/libpsm2-dev_{psm2_version}_amd64.deb',
                    'dpkg --install *.deb'
                ])

                # libibverbs
                # Available versions: http://snapshot.debian.org/binary/ibacm/
                # ibverbs_version = '21.0-1'
                # works on eve, eve has 17.2-3 installed nut this version is not available in snapshot.debian
                ib_deb_url = 'http://snapshot.debian.org/archive/debian/20180430T215634Z/pool/main'
                ibverbs_version = '17.1-2'
                ibverbs_packages = [
                    'ibacm', 'ibverbs-providers', 'ibverbs-utils',
                    'libibumad-dev', 'libibumad3', 'libibverbs-dev',
                    'libibverbs1', 'librdmacm-dev', 'librdmacm1', 'rdma-core',
                    'rdmacm-utils'
                ]
                ibverbs_cmds = ['cd /tmp']
                for package in ibverbs_packages:
                    ibverbs_cmds.extend([
                        f'wget -nv {ib_deb_url}/r/rdma-core/{package}_{ibverbs_version}_amd64.deb'
                    ])
                ibverbs_cmds.append('dpkg --install *.deb')
                Stage0 += packages(ospackages=[
                    'libnl-3-200', 'libnl-route-3-200', 'libnl-route-3-dev',
                    'udev', 'perl'
                ])
                Stage0 += shell(commands=ibverbs_cmds)

                mpicc = openmpi(
                    version=ompi,
                    cuda=False,
                    toolchain=toolchain,
                    ldconfig=True,
                    ucx='/usr/local/ucx',
                    configure_opts=[
                        '--disable-getpwuid',
                        '--sysconfdir=/mnt/0'
                        '--with-slurm',  # used on taurus
                        '--with-pmi=/usr/include/slurm-wlm',
                        'CPPFLAGS=\'-I /usr/include/slurm-wlm\'',
                        '--with-pmi-libdir=/usr/lib/x86_64-linux-gnu',
                        # '--with-pmix',
                        '--with-psm2',
                        '--disable-pty-support',
                        '--enable-mca-no-build=btl-openib,plm-slurm',
                        # eve:
                        '--with-sge',
                        '--enable-mpirun-prefix-by-default',
                        '--enable-orterun-prefix-by-default',
                    ])
            else:
                Stage0 += ucx(version='1.6.1' ,cuda=False)
                Stage0 += slurm_pmi2(version='17.02.11')
                pmix_version = True
                if version.parse(ompi) >= version.parse('4'):
                    Stage0 += pmix()
                    pmix_version = '/usr/local/pmix'

                mpicc = openmpi(version=ompi,
                                cuda=False,
                                infiniband=False,
                                pmi='/usr/local/slurm-pmi2',
                                pmix=pmix_version,
                                ucx='/usr/local/ucx')

            toolchain = mpicc.toolchain
            Stage0 += mpicc
            # OpenMPI expects this program to exist, even if it's not used.
            # Default is "ssh : rsh", but that's not installed.
            Stage0 += shell(commands=[
                'mkdir /mnt/0',
                "echo 'plm_rsh_agent = false' >> /mnt/0/openmpi-mca-params.conf"
            ])

            Stage0 += label(
                metadata={
                    'org.opengeosys.mpi': 'openmpi',
                    'org.opengeosys.mpi.version': ompi
                })

            if args.mpi_benchmarks:
                osu_app = scif(name='osu', file="_out/osu.scif")
                osu_app += osu_benchmarks(toolchain=toolchain,
                                          prefix='/scif/apps/osu')
                Stage0 += osu_app
                Stage0 += copy(src='ogscm/files/openmpi',
                               dest='/usr/local/mpi-examples')
                Stage0 += shell(commands=[
                    'mpicc -o /usr/local/bin/mpi-hello /usr/local/mpi-examples/hello.c',
                    'mpicc -o /usr/local/bin/mpi-ring /usr/local/mpi-examples/ring.c',
                    'mpicc -o /usr/local/bin/mpi-bw /usr/local/mpi-examples/bw.c',
                ])
                Stage1 += copy(_from='build',
                               src='/usr/local/bin/mpi_*',
                               dest='/usr/local/bin/')

        if ogs_version != 'off' or args.jenkins or args.gitlab:
            Stage0 += ogs_base()
        if args.gui:
            Stage0 += packages(ospackages=[
                'mesa-common-dev', 'libgl1-mesa-dev', 'libglu1-mesa-dev',
                'libxt-dev'
            ])
        if ogscm.config.g_package_manager == package_manager.CONAN:
            Stage0 += cmake(eula=True, version='3.14.7')
            conan_user_home = '/opt/conan'
            if args.dev:
                conan_user_home = ''
            Stage0 += pm_conan(user_home=conan_user_home)
            if not args.jenkins:
                Stage0 += environment(variables={'CONAN_SYSREQUIRES_SUDO': 0})
        elif ogscm.config.g_package_manager == package_manager.SPACK:
            vtk_variants = '+osmesa'
            if ompi == 'off':
                vtk_variants += ' -mpi'
            spack_packages = [
                # 'vtk@8.1.2' + vtk_variants,
                'eigen@3.3.4',
                'boost@1.68.0'
            ]
            Stage0 += pm_spack(
                packages=spack_packages,
                # ospackages=['libgl1-mesa-dev'],
                repo='https://github.com/bilke/spack',
                branch='patch-1')
            Stage0 += shell(commands=[
                '/opt/spack/bin/spack install --only dependencies vtk@8.1.2 +osmesa'
            ])
        elif ogscm.config.g_package_manager == package_manager.SYSTEM:
            Stage0 += cmake(eula=True, version='3.12.4')
            # Use ldconfig to set library search path (instead of
            # LD_LIBRARY_PATH) as host var overwrites container var. See
            # https://github.com/sylabs/singularity/pull/2669
            Stage0 += boost(version='1.66.0')  # header only?
            Stage0 += environment(variables={'BOOST_ROOT': '/usr/local/boost'})
            Stage0 += eigen()
            vtk_cmake_args = [
                '-DModule_vtkIOXML=ON',
                '-DVTK_Group_Rendering=OFF',
                '-DVTK_Group_StandAlone=OFF'
            ]
            if args.gui:
                Stage0 += packages(ospackages=[
                    'libgeotiff-dev',
                    'libqt5x11extras5-dev',
                    'libqt5xmlpatterns5-dev',
                    'libshp-dev',
                    'qt5-default'
                ])
                vtk_cmake_args = [
                    '-DVTK_BUILD_QT_DESIGNER_PLUGIN=OFF',
                    '-DVTK_Group_Qt=ON',
                    '-DVTK_QT_VERSION=5'
                ]
            if args.insitu:
                if args.gui:
                    print('--gui can not be used with --insitu!')
                    exit(1)
                Stage0 += paraview(cmake_args=['-D PARAVIEW_USE_PYTHON=ON'],
                                   edition='CATALYST',
                                   ldconfig=True,
                                   toolchain=toolchain)
            else:
                Stage0 += vtk(cmake_args=vtk_cmake_args,
                              toolchain=toolchain,
                              ldconfig=True)
            if ompi != 'off':
                Stage0 += petsc(version='3.11.3', ldconfig=True)
        if args.cvode:
            Stage0 += cvode()
        if args.cppcheck:
            Stage0 += cppcheck()
        if args.iwyy and args.compiler == 'clang':
            Stage0 += iwyy(clang_version=args.compiler_version)
        if args.docs:
            Stage0 += packages(
                ospackages=['doxygen', 'graphviz', 'texlive-base'])
        if args.gcovr:
            Stage0 += pip(pip='pip3', packages=['gcovr'])

        if args.dev:
            Stage0 += packages(ospackages=[
                'neovim', 'gdb', 'silversearcher-ag', 'ssh-client', 'less'
            ])

        if args.pip:
            Stage0 += pip(packages=args.pip, pip='pip3')
            Stage1 += pip(packages=args.pip, pip='pip3')

        if ogs_version != 'off':
            mount_args = ''
            if args.ccache:
                Stage0 += ccache(cache_size='15G')
                mount_args = f'{mount_args} --mount=type=cache,target=/opt/ccache,id=ccache'
            if args.cvode:
                cmake_args.append('-DOGS_USE_CVODE=ON')
            if args.gui:
                cmake_args.append('-DOGS_BUILD_GUI=ON')
            if args.insitu:
                cmake_args.append('-DOGS_INSITU=ON')

            Stage0 += raw(docker=f"ARG OGS_COMMIT_HASH={info.commit_hash}")

            scif_file = f"{info.out_dir}/ogs.scif"
            if info.ogsdir:
                context_path_size = len(ogs_version)
                os.chdir(ogs_version)
                mount_args = f'{mount_args} --mount=type=bind,target=/scif/apps/ogs/src,rw'
                scif_file = f"{info.out_dir[context_path_size+1:]}/ogs.scif"

            ogs_app = scif(_arguments=mount_args, name='ogs', file=scif_file)
            ogs_app += ogs(repo=info.repo,
                           branch=info.branch,
                           commit=info.commit_hash,
                           git_version=info.git_version,
                           toolchain=toolchain,
                           prefix='/scif/apps/ogs',
                           cmake_args=cmake_args,
                           parallel=args.parallel,
                           skip_lfs=True,
                           remove_build=True,
                           remove_source=True)
            Stage0 += ogs_app

        if args.jenkins:
            Stage0 += ccache(cache_size='15G')
            Stage0 += jenkins_node()

        stages_string = str(Stage0)

        if args.runtime_only:
            Stage1 += Stage0.runtime()
            if args.compiler == 'gcc' and args.compiler_version != None:
                Stage1 += packages(apt=['libstdc++6'])
            stages_string += "\n\n" + str(Stage1)

        # ---------------------------- recipe end -----------------------------

        definition_file_path = os.path.join(info.out_dir, info.definition_file)
        with open(definition_file_path, 'w') as f:
            print(stages_string, file=f)
        if args.print:
            print(stages_string)
        else:
            print(f'Created definition {definition_file_path}')

        # Create image
        if not args.build:
            continue

        if __format == 'singularity':
            subprocess.run(
                f"sudo `which singularity` build --force {info.images_out_dir}/{info.img_file}.sif"
                f"{definition_file_path}",
                shell=True)
            subprocess.run(
                f"sudo chown $USER:$USER {info.images_out_dir}/{info.img_file}.sif",
                shell=True)
            # TODO: adapt this to else
            continue

        build_cmd = (f"DOCKER_BUILDKIT=1 docker build "
                     f"-t {info.tag} -f {definition_file_path} .")
        if info.ogsdir:
            build_cmd = "(cd {0} && {1})".format(ogs_version, build_cmd)
        print(f"Running: {build_cmd}")
        subprocess.run(build_cmd, shell=True)
        inspect_out = subprocess.check_output(
            f"docker inspect {info.tag} | grep Id",
            shell=True).decode(sys.stdout.encoding)
        image_id = re.search('sha256:(\w*)', inspect_out).group(1)
        image_id_short = image_id[0:12]
        if args.upload:
            subprocess.run(f"docker push {info.tag}", shell=True)
        image_file = f'{info.images_out_dir}/{info.img_file}-{image_id_short}.sif'
        if args.convert and not os.path.exists(image_file):
            subprocess.run(
                f"cd {cwd} && singularity build --force {image_file} docker-daemon:{info.tag}",
                shell=True)

        # Deploy image
        if not args.deploy:
            continue

        deploy_config_filename = f'{cwd}/config/deploy_hosts.yml'
        if not os.path.isfile(deploy_config_filename):
            print(
                f'ERROR: {deploy_config_filename} not found but required for deploying!'
            )
            exit(1)

        with open(deploy_config_filename, 'r') as ymlfile:
            deploy_config = yaml.load(ymlfile, Loader=yaml.FullLoader)
        if not args.deploy == 'ALL' and not args.deploy in deploy_config:
            print(f'ERROR: Deploy host "{args.deploy}" not found in config!')
            exit(1)
        deploy_hosts = {}
        if args.deploy == 'ALL':
            deploy_hosts = deploy_config
        else:
            deploy_hosts[args.deploy] = deploy_config[args.deploy]
        for deploy_host in deploy_hosts:
            deploy_info = deploy_hosts[deploy_host]
            print(f'Deploying to {deploy_info} ...')
            proxy_cmd = ''
            user_cmd = ''
            if 'user' in deploy_info:
                user_cmd = f"{deploy_info['user']}@"
            if 'proxy' in deploy_info:
                proxy_cmd = f"-e 'ssh -A -J {user_cmd}{deploy_info['proxy']}'"
                print(proxy_cmd)
            print(
                subprocess.check_output(
                    f"rsync -c -v {proxy_cmd} {image_file} {user_cmd}{deploy_info['host']}:{deploy_info['dest_dir']}",
                    shell=True).decode(sys.stdout.encoding))


if __name__ == "__main__":  # pragma: no cover
    main()



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""Include-what-you-use building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.building_blocks.generic_cmake import generic_cmake


class iwyy(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm,
           hpccm.templates.tar, hpccm.templates.wget):
    """
    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/iwyy`.

    clang_version: The installed clang version. Required!

    # Examples

    ```python
    iwyy(clang_version='5.0')
    ```

    """
    def __init__(self, **kwargs):
        """Initialize building block"""
        super(iwyy, self).__init__()

        self.__baseurl = kwargs.get(
            'baseurl',
            'https://github.com/include-what-you-use/include-what-you-use/'
            'archive'
        )
        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/iwyy')
        self.__clang_version = kwargs.get('clang_version')

        self.__instructions()

    def __instructions(self):
        """String representation of the building block"""

        self += comment('Include-what-you-use for clang version {}'.format(
            self.__clang_version))
        self.__ospackages.extend([
            'libncurses5-dev', 'zlib1g-dev',
            'llvm-{}-dev'.format(self.__clang_version),
            'libclang-{}-dev'.format(self.__clang_version)
        ])
        self += packages(ospackages=self.__ospackages)
        self += generic_cmake(
            cmake_opts=[
                '-D IWYU_LLVM_ROOT_PATH=/usr/lib/llvm-{}'.format(
                    self.__clang_version)
            ],
            directory="include-what-you-use-clang_{0}.0".format(
                self.__clang_version),
            prefix=self.__prefix,
            url='{0}/clang_{1}.0.tar.gz'.format(self.__baseurl,
                                                self.__clang_version))
        self += environment(
            variables={'PATH': '{0}/bin:$PATH'.format(self.__prefix)})

        # No runtime



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""paraview building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import re
import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_cmake import generic_cmake
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.toolchain import toolchain
from hpccm.primitives.shell import shell


class paraview(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.ldconfig,
          hpccm.templates.rm, hpccm.templates.tar, hpccm.templates.wget):
    """The `paraview` building block downloads and installs the
    [paraview](https://paraview.org/) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/paraview`.

    version: The version of paraview source to download.  The default
    value is `master`.

    # Examples

    ```python
    paraview()
    ```

    """
    def __init__(self, **kwargs):
        super(paraview, self).__init__(**kwargs)

        self.__cmake_args = kwargs.get('cmake_args', [])
        self.__edition = kwargs.get('CANONICAL')
        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/paraview')
        self.__shared = kwargs.get('shared', True)
        self.__toolchain = kwargs.get('toolchain', toolchain())
        self.__version = kwargs.get('version', 'master')

        # TODO:
        if False:
            match = re.match(r'(?P<major>\d+)\.(?P<minor>\d+)\.(?P<revision>\d+)',
                             self.__version)
            short_version = '{0}.{1}'.format(match.groupdict()['major'],
                                             match.groupdict()['minor'])
            self.__baseurl = kwargs.get(
                'baseurl',
                'https://www.paraview.org/files/release/{0}'.format(short_version))
        self.__environment_variables = {}

        self.__instructions()

    def __instructions(self):
        self += comment('paraview {}'.format(self.__version))

        # if ubuntu:
        self.__ospackages.append('ninja-build')
        self += packages(ospackages=self.__ospackages)

        self.__cmake_args.extend([
            '-G Ninja',
            '-D CMAKE_BUILD_TYPE=Release',
            '-D PARAVIEW_BUILD_EDITION={}'.format(self.__edition)])
        if not self.__shared:
            self.__cmake_args.append('-D BUILD_SHARED_LIBS=OFF')
        if self.__toolchain.CC == 'mpicc':
            self.__cmake_args.append('-D PARAVIEW_USE_MPI=ON')

        # TODO: Install dependencies for rendering editions (ospackages)

        self += generic_cmake(branch = self.__version,
                              cmake_opts=self.__cmake_args,
                              directory='paraview-{}'.format(self.__version),
                              prefix=self.__prefix,
                              toolchain=self.__toolchain,
                              recursive=True,
                              repository='https://gitlab.kitware.com/paraview/paraview.git')
        self.__environment_variables['ParaView_DIR'] = self.__prefix
        # Set library path
        if self.__shared:
            libpath = os.path.join(self.__prefix, 'lib')
            if self.ldconfig:
                self += shell(commands=[self.ldcache_step(directory=libpath)])
            else:
                self.__environment_variables[
                    'LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(libpath)

        self += environment(variables=self.__environment_variables)

    def runtime(self, _from='0'):
        if not self.__shared:
            return str(comment('ParaView (empty)'))
        instructions = []
        instructions.append(comment('ParaView {}'.format(self.__version)))
        instructions.append(
            copy(_from=_from, src=self.__prefix, dest=self.__prefix))
        if self.ldconfig:
            libpath = os.path.join(self.__prefix, 'lib')
            instructions.append(
                shell(commands=[self.ldcache_step(directory=libpath)]))

        instructions.append(
            environment(variables=self.__environment_variables))
        return '\n'.join(str(x) for x in instructions)



#!/usr/bin/env python
import argparse
import hpccm
import os
import subprocess

from ogscm.building_blocks import jenkins_node

cli = argparse.ArgumentParser()
cli.add_argument("--out", type=str, default="_out")
args = cli.parse_args()


for image in ['clang7', 'gcc7', 'gcc8', 'gcc9']:
    Stage0 = hpccm.Stage()
    Stage0 += hpccm.primitives.baseimage(image='conanio/{0}'.format(image))
    Stage0 += hpccm.primitives.user(user='root')
    Stage0 += hpccm.building_blocks.pip(packages=['bincrafters-package-tools'],
                                        pip='pip3')
    Stage0 += jenkins_node()

    definition_file = 'Dockerfile.{0}'.format(image)
    definition_file_path = os.path.join(args.out, definition_file)
    with open(definition_file_path, 'w') as f:
        print(Stage0, file=f)
    tag = f"ogs6/conan_{image}"
    build_cmd = (f"docker build -t {tag} -f {definition_file_path} {args.out}")
    print(f"Running: {build_cmd}")
    subprocess.run(build_cmd, shell=True)

    subprocess.run(f"docker push {tag}", shell=True)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""Package manager Conan building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import hpccm.config

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.building_blocks.pip import pip
from hpccm.common import linux_distro
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell


class pm_conan(bb_base):
    """Package manager Conan building block"""

    def __init__(self, **kwargs):
        super(pm_conan, self).__init__()

        self.__user_home = kwargs.get('user_home', '')

        self.__commands = []

        self.__setup()

        self.__instructions()

    def __instructions(self):
        self += comment(__doc__, reformat=False)
        # https://github.com/bincrafters/community/issues/880
        self += packages(ospackages=['pkg-config'])
        # For building curl:
        self += packages(ospackages=['autoconf-archive', 'libtool'])
        conan_version = "1.22.2"

        self += pip(pip='pip3', packages=['conan=={}'.format(conan_version)])
        self += shell(commands=self.__commands)
        if self.__user_home != '':
            self += environment(variables={'CONAN_USER_HOME': self.__user_home})
        self += label(metadata={
            'org.opengeosys.pm': 'conan',
            'org.opengeosys.pm.conan.version': conan_version
        })
        if self.__user_home != '':
            self += label(metadata={
                'org.opengeosys.pm.conan.user_home': self.__user_home
            })

    def __setup(self):
        if self.__user_home != '':
            self.__commands.extend([
                # Create Conan cache dir writable by all users
                # TODO: does not work in Singularity: Read-only file system
                'mkdir -p /opt/conan',
                'chmod 777 /opt/conan'
            ])

    # No runtime



from __future__ import absolute_import

from ogscm.building_blocks.ccache import ccache
from ogscm.building_blocks.cppcheck import cppcheck
from ogscm.building_blocks.cvode import cvode
from ogscm.building_blocks.eigen import eigen
from ogscm.building_blocks.iwyy import iwyy
from ogscm.building_blocks.jenkins_node import jenkins_node
from ogscm.building_blocks.lmod import lmod
from ogscm.building_blocks.ogs import ogs
from ogscm.building_blocks.ogs_base import ogs_base
from ogscm.building_blocks.osu_benchmarks import osu_benchmarks
from ogscm.building_blocks.paraview import paraview
from ogscm.building_blocks.petsc import petsc
from ogscm.building_blocks.pm_conan import pm_conan
from ogscm.building_blocks.pm_easybuild import pm_easybuild
from ogscm.building_blocks.pm_spack import pm_spack
from ogscm.building_blocks.vtk import vtk
